{% extends "base.j2" %}

{% block body %}
{{ super() }}
<div class="row">
  <div class="col-sm-12">
    {% include "fragments/jupyter.j2" %}

    <div id="_content" class="container loading">
      <div class="text-center"><div class="lds-ellipsis"><div></div><div></div><div></div><div></div></div></div>
      <script>
        require(['svelte'], function(svelte) {
          svelte.render(
            document.getElementById("_content"),
            '{{ url_for('static', filename='js/landing.js') }}',
            JSON.parse(atob('{{ ({'nbdownload': _nb, 'extras': _config.EXTRAS, 'debug': _config.DEBUG })|jsonify|btoa }}'))
          )
        })
      </script>
    </div>
  </div>
</div>
{% endblock %}
