<div class="ipynb-field">
    <div class="form-group">
        <p>{{ this.args.description }}</p>
        <label for="{{ this.args.name }}">
            {{ this.args.label }}
        </label>
        <div class="input-group">
            <input type="text" name="{{ this.args.name }}" class="form-control" value="{{ this.args.default }}">
        </div>
    </div>
</div>