{% extends "default/base.j2" %}

{% block style %}
{{ super() }}
<link href="https://fonts.googleapis.com/css?family=Nunito:200,300,400" rel="stylesheet">
<link rel="stylesheet" href="{{ url_for('static', filename='css/biojupies.css') }}" />
<link rel="stylesheet" href="{{ url_for('static', filename='css/lib/fontawesome/css/all.min.css') }}" />
{% endblock %}

{% block script_head %}
{{ super() }}
<script>
  require.config({
    paths: {
      'bootstrap': {{ url_for('static', filename='js/lib/bootstrap/bootstrap.bundle.min')|jsonify }},
    },
    shim: {
      'bootstrap': ['jquery'],
    }
  })
</script>
{% endblock %}

{% block body %}
{{ super() }}
{% endblock %}

{% block script %}
{{ super() }}
<script>
require(['jquery', 'bootstrap'], function ($) {
  $(document).ready(function () {
    $('.dropdown-toggle').dropdown()
    $('.collapse').collapse({ toggle: false })
    $('[data-toggle="tooltip"]').tooltip({ boundary: 'window' })
  })
})
</script>
{% endblock %}
