<div id={{ this.args.name|jsonify }} class="loading">
  <div class="text-center"><div class="lds-ellipsis"><div></div><div></div><div></div><div></div></div></div>
  <script>
    require(['svelte'], function(svelte) {
      svelte.render(
        document.getElementById({{ this.args.name|jsonify }}),
        {{ this.js_url|jsonify }},
        JSON.parse(atob('{{ this|jsonify|btoa }}'))
      )
    })
  </script>
</div>