<div class="row px-4 px-lg-3 pb-4">
  <div class="col-lg-3 bold text-lg-right my-auto">
    {{ this.args.label }}
    {%- if this.args.description %}
      <sup data-toggle="tooltip" title="{{ this.args.description }}"><i class="far fa-question-circle"></i></sup>
    {% endif -%}:
  </div>
  <div class="col-lg-6 pt-2 pt-lg-0">
    <input
      type="number"
      name="{{ this.args.name }}"
      class="form-control nodecoration tiny bg-white px-2 py-1 mb-0"
      placeholder="{{ this.args.hint }}"
      step="{{ this.args.step or 0.1 }}"
      value="{{ this.args.default }}"
    />
  </div>
</div>