{% extends "default/base.j2" %}

{% block style %}
{{ super() }}
<link rel="stylesheet" href="{{ url_for('static', filename='css/bootstrap.css') }}" />
{% endblock %}

{% block script_head %}
{{ super() }}
<script>
  require.config({
    paths: {
      'bootstrap': {{ url_for('static', filename='js/lib/bootstrap/bootstrap.bundle.min')|jsonify }},
      'select2': {{ url_for('static', filename='js/lib/select2/select2.min')|jsonify }},
    },
    shim: {
      'bootstrap': ['jquery'],
      'select2': ['jquery'],
    }
  })
</script>
{% endblock %}

{% block body %}
{{ super() }}
{% endblock %}

{% block script %}
{{ super() }}
{% endblock %}
