{% extends "default/form.j2" %}

{% block script %}
{{ super() }}
<script>
require(['jquery', 'bootstrap', 'select2'], function ($) {
  $(document).ready(function () {
    $('.collapse').collapse({ toggle: false })
    $('[data-toggle="tooltip"]').tooltip({ boundary: 'window' })
    $('.select2').select2()
  })
})
</script>
{% endblock %}
