<div class="ipynb-field">
  <div class="form-group">
    {%- if this.args.description %}
      <p>{{ this.args.description }}</p>
    {% endif -%}
    <label for="{{ this.args.name }}">
      {{ this.args.label }}
    </label>
    <textarea
      class="form-control"
      name="{{ this.args.name }}"
      placeholder="{{ this.args.hint }}"
      rows="{{ this.args.rows }}"
      cols="{{ this.args.cols }}"
    >{{ this.args.default }}</textarea>
  </div>
</div>