<div class="row px-4 px-lg-3 pb-3">
  <div class="col-lg-3 bold text-lg-right my-auto">
    {{ this.args.label }}
    {%- if this.args.description %}
      <sup data-toggle="tooltip" title="{{ this.args.description }}"><i class="far fa-question-circle"></i></sup>
    {% endif -%}
  </div>
  <div class="col-lg-1 pt-1 pt-lg-0">
    <div class="border-grey rounded d-table mt-lg-1 overflow-hidden my-auto">
      <input type="radio" id="{{ this.args.name }}-yes" name="{{ this.args.name }}" value="yes" class="d-none" {% if this.value %} checked{% endif %}>
      <label for="{{ this.args.name }}-yes" class="radio-label px-2 py-1 cursor-pointer d-table-cell">Yes</label>
      <input type="radio" id="{{ this.args.name }}-no" name="{{ this.args.name }}" value="no" class="d-none" {% if not this.value %} checked{% endif %}>
      <label for="{{ this.args.name }}-no" class="radio-label px-2 py-1 cursor-pointer d-table-cell">No</label>
    </div>
  </div>
</div>